/********************************************************************
 *																	*
 * Software Name:  DSRT_NT											*
 * PLatform:       WIndows 2000										*
 * Version Number: 2.1												*
 * Release Date:   01/28/2001										*
 *																	*
 * Copyright:														*
 *		Monet Group,												*
 *		http://cairo.cs.uiuc.edu/									*
 *		Department of Computer Science,								*
 *		University of Illinois at Urbana Champaign.					*
 *																	*
 * Contacting Authors:												*
 *   Wanghong Yuan, Klara Nahrstedt {wyuan1,klara}@cs.uiuc.edu)		*
 *																	* 
 ********************************************************************/

// CPUApi.cpp: implementation of the CCPUApi class.
//
//////////////////////////////////////////////////////////////////////

#include "CPUApi.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CCPUApi::CCPUApi()	//CPUApi and RThread are in same thread
{
	m_dwThreadId		= GetCurrentThreadId();
	m_hYieldSemaAtApi	= CreateSemaphore(NULL, 0, 1, NULL);
	m_hContiSemaAtApi	= CreateSemaphore(NULL, 0, 1, NULL);

	//dupilcate the handle to m_hThreadAtApi
	HANDLE hThreadAtApi;	//handle of the RT thread in the context of CPUApi
	if(!DuplicateHandle(
		GetCurrentProcess(),	// handle to source process
		GetCurrentThread(),		// handle to duplicate
		GetCurrentProcess(),	// handle to target process
		&hThreadAtApi,			// duplicate handle
		0,						// ignore with DUPLICATE_SAME_ACCESS
		FALSE, 
		DUPLICATE_SAME_ACCESS)){
		ErrorMsg("DuplicateHandle");
	}

	//send init msg
	CMessage msg(CMD_INIT, 
		m_dwThreadId,			//RT thread id
		GetCurrentProcessId(),	//RT process id
		hThreadAtApi,			//RT thread handle
		m_hYieldSemaAtApi,		//handle of yield semaphore
		m_hContiSemaAtApi,		//handle of continue sema
		GetCurrentProcessId()	//CPUApi process id
		);

	if(!sendRecvMsg(msg)){
		ErrorMsg("sendRecvMsg");
		exit(0);
	}
}

CCPUApi::CCPUApi(DWORD dwPid, DWORD dwThrdId, HANDLE hThrdAtCln) //for other thread identified by param
{

	m_dwThreadId		= dwThrdId;
	m_hYieldSemaAtApi	= CreateSemaphore(NULL, 0, 1, NULL);
	m_hContiSemaAtApi	= CreateSemaphore(NULL, 0, 1, NULL);

	//dupilcate the handle of RThread to m_hThreadAtApi
	HANDLE hThreadAtApi;	//handle of the RT thread in the context of CPUApi
	if(!DuplicateHandle(
		OpenProcess(PROCESS_ALL_ACCESS, FALSE, dwPid), // handle to source process
		hThrdAtCln,				// handle to duplicate
		GetCurrentProcess(),	// handle to target process
		&hThreadAtApi,		// duplicate handle
		0,						// ignore with DUPLICATE_SAME_ACCESS
		FALSE, 
		DUPLICATE_SAME_ACCESS)){
		ErrorMsg("DuplicateHandle");
	}

	//send init msg
	CMessage msg(CMD_INIT, 
		m_dwThreadId,			//RT thread id
		dwPid,					//RT process id
		hThreadAtApi,			//RT thread handle
		m_hYieldSemaAtApi,		//handle of yield semaphore
		m_hContiSemaAtApi,		//handle of continue sema
		GetCurrentProcessId()	//CPUApi process id
		);

	if(!sendRecvMsg(msg))
		ErrorMsg("sendRecvMsg");
}

CCPUApi::~CCPUApi()
{
	CloseHandle(m_hYieldSemaAtApi);
	CloseHandle(m_hContiSemaAtApi);
}

inline BOOL CCPUApi::sendRecvMsg(CMessage& msg)
{
	CMsgQueue::SendReceiveMsg(msg);
	if(!msg.m_bSuccess)
		ErrorMsg(msg.MsgCmdToString());
	msg.print();
	return msg.m_bSuccess;
}

BOOL CCPUApi::Probe(LONG lPeriod)
{
	CMessage msg(CMD_PROBE, m_dwThreadId, lPeriod);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::ProbeEnd(CCPUReserve& resv)
{
	CMessage msg(CMD_PROBE_END, m_dwThreadId);
	BOOL bSuccess = sendRecvMsg(msg);
	resv = msg.m_cpuReserve;
	return bSuccess;
}

BOOL CCPUApi::Reserve(CCPUReserve resv)
{
	CMessage msg(CMD_RESERVE, m_dwThreadId, resv);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::StartRTRun()
{
	CMessage msg(CMD_START, m_dwThreadId);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::StopRTRun()
{
	CMessage msg(CMD_STOP, m_dwThreadId);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::KillRThread()
{
	CMessage msg(CMD_KILL, m_dwThreadId);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::YieldCPU()
{
	ReleaseSemaphore(m_hYieldSemaAtApi, 1, NULL); //wake up server
	WaitForSingleObject(m_hContiSemaAtApi, INFINITE); //wait until released by server
	return TRUE;
}

BOOL CCPUApi::UpdateReservation(CCPUReserve resv)
{
	CMessage msg(CMD_UPDATE, m_dwThreadId, resv);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::FreeReservation()
{
	CMessage msg(CMD_FREE, m_dwThreadId);
	return sendRecvMsg(msg);
}

BOOL CCPUApi::KillServer()
{
	CMessage msg(CMD_KILLSERVER, m_dwThreadId);
	return sendRecvMsg(msg);
}

